@@ -773,15 +773,13 @@ transpose_critical_parameters (struct jpeg_compress_struct* dstinfo)
  * This must be called after jpeg_copy_critical_parameters ()
  * and before jpeg_write_coefficients ().
  * The code has been adapted from transupp.c.
- * @param srcinfo		source image information
  * @param dstinfo		destination image information
  * @param src_coef_arrays	coefficient arrays for the source image
  * @param info			transformation information
  * @return	virtual coefficient arrays for jpeg_write_coefficients ()
  */
 static struct jvirt_barray_control**
-jtransform_adjust_parameters (struct jpeg_decompress_struct* srcinfo,
-			      struct jpeg_compress_struct* dstinfo,
+jtransform_adjust_parameters (struct jpeg_compress_struct* dstinfo,
 			      struct jvirt_barray_control** src_coef_arrays,
 			      struct jpeg_transform_info* info)
 {
@@ -798,6 +796,7 @@ jtransform_adjust_parameters (struct jpeg_decompress_struct* srcinfo,
   case JXFORM_ROT_90:
   case JXFORM_ROT_270:
     transpose_critical_parameters (dstinfo);
+    /*@fallthrough@*/
   case JXFORM_NONE:
   case JXFORM_FLIP_H:
   case JXFORM_FLIP_V:
@@ -897,32 +896,32 @@ jcopy_markers_execute (struct jpeg_decompress_struct* srcinfo,
    */
   for (marker = srcinfo->marker_list; marker != NULL; marker = marker->next) {
     if (dstinfo->write_JFIF_header &&
-	marker->marker == JPEG_APP0 &&
+	marker->marker == (UINT8) JPEG_APP0 &&
 	marker->data_length >= 5 &&
-	GETJOCTET (marker->data[0]) == 0x4A &&
-	GETJOCTET (marker->data[1]) == 0x46 &&
-	GETJOCTET (marker->data[2]) == 0x49 &&
-	GETJOCTET (marker->data[3]) == 0x46 &&
-	GETJOCTET (marker->data[4]) == 0)
+	GETJOCTET (marker->data[0]) == (JOCTET) 0x4A &&
+	GETJOCTET (marker->data[1]) == (JOCTET) 0x46 &&
+	GETJOCTET (marker->data[2]) == (JOCTET) 0x49 &&
+	GETJOCTET (marker->data[3]) == (JOCTET) 0x46 &&
+	GETJOCTET (marker->data[4]) == (JOCTET) 0)
       continue;			/* reject duplicate JFIF */
     if (dstinfo->write_Adobe_marker &&
-	marker->marker == JPEG_APP0 + 14 &&
+	marker->marker == (UINT8) (JPEG_APP0 + 14) &&
 	marker->data_length >= 5 &&
-	GETJOCTET (marker->data[0]) == 0x41 &&
-	GETJOCTET (marker->data[1]) == 0x64 &&
-	GETJOCTET (marker->data[2]) == 0x6F &&
-	GETJOCTET (marker->data[3]) == 0x62 &&
-	GETJOCTET (marker->data[4]) == 0x65)
+	GETJOCTET (marker->data[0]) == (JOCTET) 0x41 &&
+	GETJOCTET (marker->data[1]) == (JOCTET) 0x64 &&
+	GETJOCTET (marker->data[2]) == (JOCTET) 0x6F &&
+	GETJOCTET (marker->data[3]) == (JOCTET) 0x62 &&
+	GETJOCTET (marker->data[4]) == (JOCTET) 0x65)
       continue;			/* reject duplicate Adobe */
     if (length &&
-	marker->marker == JPEG_APP0 + 1 &&
+	marker->marker == (UINT8) (JPEG_APP0 + 1) &&
 	marker->data_length >= 8 &&
-	GETJOCTET (marker->data[0]) == 0x45 &&
-	GETJOCTET (marker->data[1]) == 0x78 &&
-	GETJOCTET (marker->data[2]) == 0x69 &&
-	GETJOCTET (marker->data[3]) == 0x66 &&
-	!GETJOCTET (marker->data[4]) &&
-	!GETJOCTET (marker->data[5])) {
+	GETJOCTET (marker->data[0]) == (JOCTET) 0x45 &&
+	GETJOCTET (marker->data[1]) == (JOCTET) 0x78 &&
+	GETJOCTET (marker->data[2]) == (JOCTET) 0x69 &&
+	GETJOCTET (marker->data[3]) == (JOCTET) 0x66 &&
+	GETJOCTET (marker->data[4]) == (JOCTET) 0x00 &&
+	GETJOCTET (marker->data[5]) == (JOCTET) 0x00) {
       unsigned i;
       /* writing Exif marker with embedded thumbnail */
       jpeg_write_m_header (dstinfo, marker->marker, marker->data_length +
@@ -961,7 +960,7 @@ touchfile (const char* name,
   struct utimbuf utim;
   utim.actime = utim.modtime = datetime;
   if (utime (name, &utim)) {
-    fputs (name, stderr);
+    (void) fputs (name, stderr);
     perror (": utime");
     /* non-fatal error, don't abort */
   }
@@ -987,7 +986,7 @@ exif_error_exit (struct jpeg_common_struct* cinfo)
 }
 
 /** current input file name */
-static const char* inputfile;
+static const char* srcname;
 
 /**
  * Custom message output handler
@@ -999,10 +998,10 @@ exif_output_message (struct jpeg_common_struct* cinfo)
   /** error message buffer for libjpeg */
   static char jpeg_msg[JMSG_LENGTH_MAX];
   cinfo->err->format_message (cinfo, jpeg_msg);
-  fputs (inputfile, stderr);
-  fputs (": ", stderr);
-  fputs (jpeg_msg, stderr);
-  putc ('\n', stderr);
+  (void) fputs (srcname, stderr);
+  (void) fputs (": ", stderr);
+  (void) fputs (jpeg_msg, stderr);
+  (void) putc ('\n', stderr);
 }
 
 /** error messages from the Exif parser */
@@ -1112,7 +1111,11 @@ thumb_in_fill (struct jpeg_decompress_struct* cinfo)
  * @param cinfo	the compression context
  */
 static void
-thumb_out_dummy (struct jpeg_compress_struct* cinfo)
+thumb_out_dummy (struct jpeg_compress_struct* cinfo
+#ifdef __GNUC__
+		 __attribute__ ((unused))
+#endif /* __GNUC__ */
+		 )
 {
 }
 
@@ -1130,14 +1133,16 @@ thumb_out_empty (struct jpeg_compress_struct* cinfo)
 
 /** thumbnail compression buffer */
 static JOCTET thumb_buf[16384];
-/** thumbnail length */
+/** actual thumbnail length */
 static unsigned thumb_length;
+/** maximum thumbnail length */
+static unsigned thumb_maxlen;
 
 /** transform a thumbnail image in memory
  * @param xform	transformation
  * @param data	image data
- * @param len	image data length
- * @param maxl	maximum image data length
+ * @param len	source image data length
+ * @param maxl	maximum output image data length
  */
 static void
 thumb_transform (enum JXFORM_CODE xform,
@@ -1155,6 +1160,10 @@ thumb_transform (enum JXFORM_CODE xform,
   struct jvirt_barray_control** src_coef_arrays;
   struct jvirt_barray_control** dst_coef_arrays;
 
+  thumb_maxlen = maxl < (sizeof thumb_buf) / sizeof *thumb_buf
+    ? (sizeof thumb_buf) / sizeof *thumb_buf
+    : maxl;
+
   srcinfo.err = dstinfo.err = jpeg_std_error (&jerr.pub);
   jpeg_create_decompress (&srcinfo);
   jpeg_create_compress (&dstinfo);
@@ -1176,13 +1185,13 @@ thumb_transform (enum JXFORM_CODE xform,
   srcinfo.src->next_input_byte = data;
   srcinfo.src->bytes_in_buffer = len;
   srcinfo.src->fill_input_buffer = thumb_in_fill;
-  jpeg_read_header (&srcinfo, TRUE);
+  (void) jpeg_read_header (&srcinfo, TRUE);
 
   info.transform = xform;
   jtransform_request_workspace (&srcinfo, &info);
   src_coef_arrays = jpeg_read_coefficients (&srcinfo);
   jpeg_copy_critical_parameters (&srcinfo, &dstinfo);
-  dst_coef_arrays = jtransform_adjust_parameters (&srcinfo, &dstinfo,
+  dst_coef_arrays = jtransform_adjust_parameters (&dstinfo,
 						  src_coef_arrays,
 						  &info);
 
@@ -1191,16 +1200,14 @@ thumb_transform (enum JXFORM_CODE xform,
     dstinfo.dest->term_destination = thumb_out_dummy;
   dstinfo.dest->empty_output_buffer = thumb_out_empty;
   dstinfo.dest->next_output_byte = thumb_buf;
-  if (maxl < (sizeof thumb_buf) / sizeof *thumb_buf)
-    maxl = (sizeof thumb_buf) / sizeof *thumb_buf;
-  dstinfo.dest->free_in_buffer = maxl;
+  dstinfo.dest->free_in_buffer = thumb_maxlen;
   dstinfo.optimize_coding = TRUE;
   dstinfo.write_JFIF_header = FALSE;
   jpeg_write_coefficients (&dstinfo, dst_coef_arrays);
   jtransform_execute_transform (&srcinfo, &dstinfo, src_coef_arrays, &info);
   (void) jpeg_finish_compress (&dstinfo);
   (void) jpeg_finish_decompress (&srcinfo);
-  thumb_length = maxl - dstinfo.dest->free_in_buffer;
+  thumb_length = thumb_maxlen - dstinfo.dest->free_in_buffer;
  done:
   jpeg_destroy_compress (&dstinfo);
   jpeg_destroy_decompress (&srcinfo);
@@ -1222,8 +1229,8 @@ exif_read2 (JOCTET FAR* data,
 	    enum endianness endian)
 {
   return endian
-    ? GETJOCTET (data[0]) << 8 | GETJOCTET (data[1])
-    : GETJOCTET (data[0]) | GETJOCTET (data[1]) << 8;
+    ? (unsigned) GETJOCTET (data[0]) << 8 | GETJOCTET (data[1])
+    : (unsigned) GETJOCTET (data[0]) | GETJOCTET (data[1]) << 8;
 }
 
 /** Read a 4-octet quantity from the Exif record
@@ -1255,33 +1262,36 @@ exif_date (JOCTET FAR* date)
 {
   struct tm stamp;
   time_t t;
-  unsigned year, month, day, h, m, s;
+  int year, month, day, h, m, s;
   unsigned i;
   if (date[19])
     return -1; /* not NUL-terminated */
   for (i = 19; i--; ) {
     switch (i) {
     case 4: case 7: case 13: case 16:
-      if (date[i] != 0x3a/*':'*/)
+      if (date[i] != (JOCTET) 0x3a/*':'*/)
 	return -1; /* improper field terminator */
       break;
     case 10:
-      if (date[i] != 0x20/*' '*/)
+      if (date[i] != (JOCTET) 0x20/*' '*/)
 	return -1; /* date and time not separated by space */
       break;
     default:
-      if (date[i] < 0x30/*'0'*/ || date[i] > 0x39/*'9'*/)
+      if (date[i] < (JOCTET) 0x30/*'0'*/ || date[i] > (JOCTET) 0x39/*'9'*/)
 	return -1; /* non-digits in date or time */
     }
   }
   year =
-    (date[0] - 0x30/*'0'*/) * 1000 + (date[1] - 0x30/*'0'*/) * 100 +
-    (date[2] - 0x30/*'0'*/) * 10 + (date[3] - 0x30/*'0'*/) - 1900;
-  month = (date[5] - 0x30/*'0'*/) * 10 + (date[6] - 0x30/*'0'*/) - 1;
-  day = (date[8] - 0x30/*'0'*/) * 10 + (date[9] - 0x30/*'0'*/);
-  h = (date[11] - 0x30/*'0'*/) * 10 + (date[12] - 0x30/*'0'*/);
-  m = (date[14] - 0x30/*'0'*/) * 10 + (date[15] - 0x30/*'0'*/);
-  s = (date[17] - 0x30/*'0'*/) * 10 + (date[18] - 0x30/*'0'*/);
+    ((int) date[0] - 0x30/*'0'*/) * 1000 +
+    ((int) date[1] - 0x30/*'0'*/) * 100 +
+    ((int) date[2] - 0x30/*'0'*/) * 10 +
+    ((int) date[3] - 0x30/*'0'*/) - 1900;
+  month = ((int) date[5] - 0x30/*'0'*/) * 10 +
+    ((int) date[6] - 0x30/*'0'*/) - 1;
+  day = ((int) date[8] - 0x30/*'0'*/) * 10 + ((int) date[9] - 0x30/*'0'*/);
+  h = ((int) date[11] - 0x30/*'0'*/) * 10 + ((int) date[12] - 0x30/*'0'*/);
+  m = ((int) date[14] - 0x30/*'0'*/) * 10 + ((int) date[15] - 0x30/*'0'*/);
+  s = ((int) date[17] - 0x30/*'0'*/) * 10 + ((int) date[18] - 0x30/*'0'*/);
   stamp.tm_sec = s; stamp.tm_min = m; stamp.tm_hour = h;
   stamp.tm_mday = day; stamp.tm_mon = month; stamp.tm_year = year;
   stamp.tm_wday = stamp.tm_yday = stamp.tm_isdst = -1;
@@ -1308,7 +1318,7 @@ exif_parse (struct jpeg_decompress_struct* cinfo,
 	    enum JXFORM_CODE xform,
 	    time_t* timestamp,
 	    int rmifd1,
-	    char* thumbname)
+	    const char* thumbname)
 {
   jpeg_saved_marker_ptr marker;
 
@@ -1318,27 +1328,27 @@ exif_parse (struct jpeg_decompress_struct* cinfo,
     /** current image file directory (IFD) offset */
     unsigned long offset;
 
-    if (marker->marker != JPEG_APP0 + 1 ||
+    if (marker->marker != (JOCTET) (JPEG_APP0 + 1) ||
 	marker->data_length < 14)
       continue; /* Exif data is in an APP1 marker of at least 14 octets */
 
-    if (GETJOCTET (marker->data[0]) != 0x45 ||
-	GETJOCTET (marker->data[1]) != 0x78 ||
-	GETJOCTET (marker->data[2]) != 0x69 ||
-	GETJOCTET (marker->data[3]) != 0x66 ||
-	GETJOCTET (marker->data[4]) ||
-	GETJOCTET (marker->data[5]))
+    if (GETJOCTET (marker->data[0]) != (JOCTET) 0x45 ||
+	GETJOCTET (marker->data[1]) != (JOCTET) 0x78 ||
+	GETJOCTET (marker->data[2]) != (JOCTET) 0x69 ||
+	GETJOCTET (marker->data[3]) != (JOCTET) 0x66 ||
+	GETJOCTET (marker->data[4]) != (JOCTET) 0x00 ||
+	GETJOCTET (marker->data[5]) != (JOCTET) 0x00)
       continue; /* no Exif header */
 
-    if (GETJOCTET (marker->data[6]) == 0x4d &&
-	GETJOCTET (marker->data[7]) == 0x4d &&
-	GETJOCTET (marker->data[8]) == 0x00 &&
-	GETJOCTET (marker->data[9]) == 0x2a)
+    if (GETJOCTET (marker->data[6]) == (JOCTET) 0x4d &&
+	GETJOCTET (marker->data[7]) == (JOCTET) 0x4d &&
+	GETJOCTET (marker->data[8]) == (JOCTET) 0x00 &&
+	GETJOCTET (marker->data[9]) == (JOCTET) 0x2a)
       endian = MSB;
-    else if (GETJOCTET (marker->data[6]) == 0x49 &&
-	     GETJOCTET (marker->data[7]) == 0x49 &&
-	     GETJOCTET (marker->data[8]) == 0x2a &&
-	     GETJOCTET (marker->data[9]) == 0x00)
+    else if (GETJOCTET (marker->data[6]) == (JOCTET) 0x49 &&
+	     GETJOCTET (marker->data[7]) == (JOCTET) 0x49 &&
+	     GETJOCTET (marker->data[8]) == (JOCTET) 0x2a &&
+	     GETJOCTET (marker->data[9]) == (JOCTET) 0x00)
       endian = LSB;
     else
       ERREXIT (cinfo, EXIF_UNKNOWN_ENDIAN);
@@ -1405,7 +1415,7 @@ exif_parse (struct jpeg_decompress_struct* cinfo,
       ifdsize = exif_read2 (&exif_data[offset], endian);
       offset += 2;
       /* Get the IFD1 offset (thumbnail image) */
-      ifd1_offset_ptr = ifdend = offset + 12 * ifdsize;
+      ifd1_offset_ptr = ifdend = (unsigned) offset + 12 * ifdsize;
       if (ifdend > exif_length - 4)
 	ERREXIT (cinfo, EXIF_IFD0_EOF);
       ifd1_offset = exif_read4 (&exif_data[ifdend], endian);
@@ -1430,7 +1440,7 @@ exif_parse (struct jpeg_decompress_struct* cinfo,
 		(string_offset < ifdend ||
 		 string_offset >
 		 (ifd1_offset ? ifd1_offset : exif_length) - string_len ||
-		 exif_data[string_offset + string_len - 1]))
+		 exif_data[string_offset + string_len - 1] != (JOCTET) 0))
 	      ERREXIT (cinfo, EXIF_STRING_INVALID);
 	  }
 	  break;
@@ -1447,22 +1457,22 @@ exif_parse (struct jpeg_decompress_struct* cinfo,
 	  /* set the orientation to 1 (top, left-hand) */
 	  switch (endian) {
 	  case LSB:
-	    exif_data[offset + 8] = 1;
+	    exif_data[offset + 8] = (JOCTET) 1;
 	    break;
 	  case MSB:
-	    exif_data[offset + 9] = 1;
+	    exif_data[offset + 9] = (JOCTET) 1;
 	    break;
 	  }
 	  break;
 	case 0x011a: /* XResolution */
 	  if (res_x0_ptr)
 	    ERREXIT (cinfo, EXIF_RESX_DUP);
-	  res_x0_ptr = offset + endian;
+	  res_x0_ptr = (unsigned) offset + endian;
 	  break;
 	case 0x011b: /* YResolution */
 	  if (res_y0_ptr)
 	    ERREXIT (cinfo, EXIF_RESY_DUP);
-	  res_y0_ptr = offset + endian;
+	  res_y0_ptr = (unsigned) offset + endian;
 	  break;
 	case 0x0132: /* DateTime */
 	  if (datetime != -1)
@@ -1494,7 +1504,7 @@ exif_parse (struct jpeg_decompress_struct* cinfo,
 	/* Get the number of SubIFD entries */
 	ifdsize = exif_read2 (&exif_data[offset], endian);
 	offset += 2;
-	ifdend = offset + 12 * ifdsize;
+	ifdend = (unsigned) offset + 12 * ifdsize;
 	if (ifdend > (ifd1_offset ? ifd1_offset : exif_length) - 4)
 	  ERREXIT (cinfo, EXIF_SUBIFD_EOF);
 	/* Get the next IFD offset (should be zero) */
@@ -1560,9 +1570,9 @@ exif_parse (struct jpeg_decompress_struct* cinfo,
 		       : EXIF_HEIGHT_INVALID);
 	    }
 	    if (tagnum == 0xa002)
-	      width_ptr = offset;
+	      width_ptr = (unsigned) offset;
 	    else
-	      height_ptr = offset;
+	      height_ptr = (unsigned) offset;
 	    break;
 	  case 0xa005: /* ExifInteroperabilityOffset */
 	    if (inop_offset)
@@ -1577,12 +1587,12 @@ exif_parse (struct jpeg_decompress_struct* cinfo,
 	  case 0xa20e: /* FocalPlaneXResolution */
 	    if (fres_x_ptr)
 	      ERREXIT (cinfo, EXIF_FRESX_DUP);
-	    fres_x_ptr = offset + endian;
+	    fres_x_ptr = (unsigned) offset + endian;
 	    break;
 	  case 0xa20f: /* FocalPlaneYResolution */
 	    if (fres_y_ptr)
 	      ERREXIT (cinfo, EXIF_FRESY_DUP);
-	    fres_y_ptr = offset + endian;
+	    fres_y_ptr = (unsigned) offset + endian;
 	    break;
 	  }
 	}
@@ -1592,7 +1602,7 @@ exif_parse (struct jpeg_decompress_struct* cinfo,
 	/* Get the number of Interoperability IFD entries */
 	ifdsize = exif_read2 (&exif_data[offset], endian);
 	offset += 2;
-	ifdend = offset + 12 * ifdsize;
+	ifdend = (unsigned) offset + 12 * ifdsize;
 	if (ifdend > exif_length - 4)
 	  ERREXIT (cinfo, EXIF_INOP_EOF);
 	/* Get the next IFD offset (should be zero) */
@@ -1616,12 +1626,12 @@ exif_parse (struct jpeg_decompress_struct* cinfo,
 	  case 0x1001: /* RelatedImageWidth */
 	    if (ri_x_ptr)
 	      ERREXIT (cinfo, EXIF_RIX_DUP);
-	    ri_x_ptr = offset + endian;
+	    ri_x_ptr = (unsigned) offset + endian;
 	    break;
 	  case 0x1002: /* RelatedImageHeight */
 	    if (ri_y_ptr)
 	      ERREXIT (cinfo, EXIF_RIY_DUP);
-	    ri_y_ptr = offset + endian;
+	    ri_y_ptr = (unsigned) offset + endian;
 	    break;
 	  }
 	}
@@ -1655,12 +1665,12 @@ exif_parse (struct jpeg_decompress_struct* cinfo,
 	  case 0x011a: /* XResolution */
 	    if (res_x1_ptr)
 	      ERREXIT (cinfo, EXIF_RESX_DUP);
-	    res_x1_ptr = offset + endian;
+	    res_x1_ptr = (unsigned) offset + endian;
 	    break;
 	  case 0x011b: /* YResolution */
 	    if (res_y1_ptr)
 	      ERREXIT (cinfo, EXIF_RESY_DUP);
-	    res_y1_ptr = offset + endian;
+	    res_y1_ptr = (unsigned) offset + endian;
 	    break;
 	  case 0x201: /* JpegIFOffset */
 	    if (thumb_offset)
@@ -1697,7 +1707,7 @@ exif_parse (struct jpeg_decompress_struct* cinfo,
 	  ERREXIT (cinfo, EXIF_THUMBNAIL_COPY);
 	if (thumb_offset + thumb_size > exif_length)
 	  thumb_size = exif_length - thumb_offset;
-	if (!exif_data[thumb_offset + thumb_size - 1])
+	if (exif_data[thumb_offset + thumb_size - 1] == (JOCTET) 0)
 	  thumb_size--; /* some cameras add a terminating NUL character */
 	/* output the thumbnail */
 	ok = thumb_size == fwrite (exif_data + thumb_offset, 1, thumb_size, f);
@@ -1710,8 +1720,8 @@ exif_parse (struct jpeg_decompress_struct* cinfo,
 	exif_data[ifd1_offset_ptr] =
 	  exif_data[ifd1_offset_ptr + 1] = 
 	  exif_data[ifd1_offset_ptr + 2] = 
-	  exif_data[ifd1_offset_ptr + 3] = 0;
-	exif_length = ifd1_offset;
+	  exif_data[ifd1_offset_ptr + 3] = (JOCTET) 0;
+	exif_length = (unsigned) ifd1_offset;
 	marker->data_length = exif_length + 6;
       }
 
@@ -1748,33 +1758,35 @@ exif_parse (struct jpeg_decompress_struct* cinfo,
 	}
 	switch (endian) {
 	case LSB:
-	  exif_data[width_ptr + 8] = width;
-	  exif_data[width_ptr + 9] = width >> 8;
-	  exif_data[width_ptr + 10] = exif_data[width_ptr + 11] = 0;
-	  exif_data[height_ptr + 8] = height;
-	  exif_data[height_ptr + 9] = height >> 8;
-	  exif_data[height_ptr + 10] = exif_data[height_ptr + 11] = 0;
+	  exif_data[width_ptr + 8] = (JOCTET) width;
+	  exif_data[width_ptr + 9] = (JOCTET) (width >> 8);
+	  exif_data[width_ptr + 10] = exif_data[width_ptr + 11] = (JOCTET) 0;
+	  exif_data[height_ptr + 8] = (JOCTET) height;
+	  exif_data[height_ptr + 9] = (JOCTET) (height >> 8);
+	  exif_data[height_ptr + 10] = exif_data[height_ptr + 11] = (JOCTET) 0;
 	  break;
 	case MSB:
-	  if (exif_data[width_ptr + 3] == 4) { /* unsigned long */
-	    exif_data[width_ptr + 11] = width;
-	    exif_data[width_ptr + 10] = width >> 8;
-	    exif_data[width_ptr + 9] = exif_data[width_ptr + 8] = 0;
+	  if (exif_data[width_ptr + 3] == (JOCTET) 4) { /* unsigned long */
+	    exif_data[width_ptr + 11] = (JOCTET) width;
+	    exif_data[width_ptr + 10] = (JOCTET) (width >> 8);
+	    exif_data[width_ptr + 9] = exif_data[width_ptr + 8] = (JOCTET) 0;
 	  }
 	  else { /* unsigned short */
-	    exif_data[width_ptr + 11] = exif_data[width_ptr + 10] = 0;
-	    exif_data[width_ptr + 9] = width;
-	    exif_data[width_ptr + 8] = width >> 8;
+	    exif_data[width_ptr + 11] = exif_data[width_ptr + 10] = (JOCTET) 0;
+	    exif_data[width_ptr + 9] = (JOCTET) width;
+	    exif_data[width_ptr + 8] = (JOCTET) (width >> 8);
 	  }
-	  if (exif_data[height_ptr + 3] == 4) { /* unsigned long */
-	    exif_data[height_ptr + 11] = height;
-	    exif_data[height_ptr + 10] = height >> 8;
-	    exif_data[height_ptr + 9] = exif_data[height_ptr + 8] = 0;
+	  if (exif_data[height_ptr + 3] == (JOCTET) 4) { /* unsigned long */
+	    exif_data[height_ptr + 11] = (JOCTET) height;
+	    exif_data[height_ptr + 10] = (JOCTET) (height >> 8);
+	    exif_data[height_ptr + 9] = exif_data[height_ptr + 8] =
+	      (JOCTET) 0;
 	  }
 	  else { /* unsigned short */
-	    exif_data[height_ptr + 11] = exif_data[height_ptr + 10] = 0;
-	    exif_data[height_ptr + 9] = height;
-	    exif_data[height_ptr + 8] = height >> 8;
+	    exif_data[height_ptr + 11] = exif_data[height_ptr + 10] =
+	      (JOCTET) 0;
+	    exif_data[height_ptr + 9] = (JOCTET) height;
+	    exif_data[height_ptr + 8] = (JOCTET) (height >> 8);
 	  }
 	  break;
 	}
@@ -1785,14 +1797,14 @@ exif_parse (struct jpeg_decompress_struct* cinfo,
       case JXFORM_TRANSVERSE:
       case JXFORM_ROT_90:
       case JXFORM_ROT_270:
-	if (ri_x_ptr) exif_data[ri_x_ptr] = 2;
-	if (ri_y_ptr) exif_data[ri_y_ptr] = 1;
-	if (res_x0_ptr) exif_data[res_x0_ptr] = 0x1b;
-	if (res_y0_ptr) exif_data[res_y0_ptr] = 0x1a;
-	if (res_x1_ptr) exif_data[res_x1_ptr] = 0x1b;
-	if (res_y1_ptr) exif_data[res_y1_ptr] = 0x1a;
-	if (fres_x_ptr) exif_data[fres_x_ptr] = 0x0f;
-	if (fres_y_ptr) exif_data[fres_y_ptr] = 0x0e;
+	if (ri_x_ptr) exif_data[ri_x_ptr] = (JOCTET) 2;
+	if (ri_y_ptr) exif_data[ri_y_ptr] = (JOCTET) 1;
+	if (res_x0_ptr) exif_data[res_x0_ptr] = (JOCTET) 0x1b;
+	if (res_y0_ptr) exif_data[res_y0_ptr] = (JOCTET) 0x1a;
+	if (res_x1_ptr) exif_data[res_x1_ptr] = (JOCTET) 0x1b;
+	if (res_y1_ptr) exif_data[res_y1_ptr] = (JOCTET) 0x1a;
+	if (fres_x_ptr) exif_data[fres_x_ptr] = (JOCTET) 0x0f;
+	if (fres_y_ptr) exif_data[fres_y_ptr] = (JOCTET) 0x0e;
 	break;
       case JXFORM_NONE:
       case JXFORM_FLIP_H:
@@ -1814,12 +1826,12 @@ exif_parse (struct jpeg_decompress_struct* cinfo,
 	  marker->data_length = exif_length + 6;
 	  switch (endian) {
 	  case LSB:
-	    exif_data[bytecount_ptr + 8] = thumb_length;
-	    exif_data[bytecount_ptr + 9] = thumb_length >> 8;
+	    exif_data[bytecount_ptr + 8] = (JOCTET) thumb_length;
+	    exif_data[bytecount_ptr + 9] = (JOCTET) (thumb_length >> 8);
 	    break;
 	  case MSB:
-	    exif_data[bytecount_ptr + 11] = thumb_length;
-	    exif_data[bytecount_ptr + 10] = thumb_length >> 8;
+	    exif_data[bytecount_ptr + 11] = (JOCTET) thumb_length;
+	    exif_data[bytecount_ptr + 10] = (JOCTET) (thumb_length >> 8);
 	    break;
 	  }
 	}
@@ -1863,6 +1875,131 @@ createfilename (const char* base,
   return fnam;
 }
 
+/** Transform the image
+ * @param srcinfo	image decompression context
+ * @param dstinfo	image compression context
+ * @param thumbname	thumbnail file name (optional)
+ * @param dstname	destination image file name (NULL = stdin)
+ * @param xform		image transformation
+ * @param rmifd1	flag: remove Exif image file directory 1 (thumbnail)
+ * @param datetime	(output) timestamp to collect from Exif block
+ * @param progressive	flag: use progressive coding
+ * @return		0=ok, 1=read error, 2=write error
+ */
+static int
+jpeg_transform (struct jpeg_decompress_struct* srcinfo,
+		struct jpeg_compress_struct* dstinfo,
+		const char* thumbname,
+		const char* dstname,
+		enum JXFORM_CODE xform,
+		int rmifd1,
+		time_t* datetime,
+		int progressive)
+{
+  /** source file handle */
+  static FILE* src = 0;
+  /** destination file handle */
+  static FILE* dst = 0;
+  /** image transformation information */
+  struct jpeg_transform_info info;
+  /** flag: is this a JFIF image (not Exif) */
+  int jfif = 0;
+  /** coefficient arrays of source image */
+  struct jvirt_barray_control** src_coef_arrays;
+  /** coefficient arrays of destination image */
+  struct jvirt_barray_control** dst_coef_arrays;
+
+  /* set up jump context for input */
+  if (setjmp (((struct exif_error_mgr*) srcinfo->err)->setjmp_buffer)) {
+    if (errno)
+      perror (dstname ? srcname : "(standard input)");
+    else
+      exif_output_message ((j_common_ptr) srcinfo);
+    jpeg_abort_decompress (srcinfo);
+    jpeg_abort_compress (dstinfo);
+    if (src) (void) fclose (src);
+    if (dst) (void) fclose (dst);
+    return 1; /* ignore unreadable or erroneous input files */
+  }
+
+  /* set up jump context for output */
+  if (setjmp (((struct exif_error_mgr*) dstinfo->err)->setjmp_buffer)) {
+    if (errno)
+      perror (dstname ? dstname : "(standard output)");
+    else
+      exif_output_message ((j_common_ptr) dstinfo);
+    jpeg_abort_decompress (srcinfo);
+    jpeg_abort_compress (dstinfo);
+    if (src) (void) fclose (src);
+    if (dst) (void) fclose (dst);
+    return 2; /* abort on write error */
+  }
+
+  if (dstname && !(src = fopen (srcname, "rb"))) {
+    (void) fputs (srcname, stderr);
+    perror (": open (reading)");
+    return 1;
+  }
+  jpeg_stdio_src (srcinfo, dstname ? src : stdin);
+  jcopy_markers_setup (srcinfo);
+  (void) jpeg_read_header (srcinfo, TRUE);
+
+  info.transform = exif_parse (srcinfo, xform, datetime, rmifd1, thumbname);
+  jfif = !info.transform;
+  if (jfif)
+    info.transform = xform ? xform : JXFORM_NONE;
+
+  jtransform_request_workspace (srcinfo, &info);
+
+  /* Read source file as DCT coefficients */
+  src_coef_arrays = jpeg_read_coefficients (srcinfo);
+  /* Close the input file */
+  if (src && fclose (src)) {
+    (void) fputs (srcname, stderr);
+    perror ("close");
+    /* ignore error on closing input file */
+  }
+  src = 0;
+  /* Initialize destination compression parameters from source values */
+  jpeg_copy_critical_parameters (srcinfo, dstinfo);
+
+  dst_coef_arrays = jtransform_adjust_parameters (dstinfo,
+						  src_coef_arrays,
+						  &info);
+  if (dstname) {
+    int fd;
+    (void) unlink (dstname);
+    fd = open (dstname,
+#if defined WIN32 || defined __WIN32
+	       O_WRONLY | O_CREAT | O_EXCL | O_BINARY, 0666
+#else
+	       O_WRONLY | O_CREAT | O_EXCL, 0444
+#endif
+	       );
+    if (fd < 0 || !(dst = fdopen (fd, "wb"))) {
+      (void) fputs (dstname, stderr);
+      perror (": open (writing)");
+      return 2;
+    }
+  }
+  jpeg_stdio_dest (dstinfo, dstname ? dst : stdout);
+  dstinfo->optimize_coding = TRUE;
+  dstinfo->write_JFIF_header = jfif;
+  if (progressive)
+    jpeg_simple_progression (dstinfo);
+  jpeg_write_coefficients (dstinfo, dst_coef_arrays);
+  jcopy_markers_execute (srcinfo, dstinfo, thumb_buf, thumb_length);
+  jtransform_execute_transform (srcinfo, dstinfo, src_coef_arrays, &info);
+  (void) jpeg_finish_compress (dstinfo);
+  (void) jpeg_finish_decompress (srcinfo);
+  if (dst && fclose (dst)) {
+    (void) fputs (dstname, stderr);
+    perror ("close");
+    return 2;
+  }
+  return 0;
+}
+
 int
 main (int argc, char **argv)
 {
@@ -1889,10 +2026,6 @@ main (int argc, char **argv)
   char* imagedir = 0;
   /** directory for Exif thumbnail files (0=output disabled) */
   char* thumbdir = 0;
-  /** imagedir length */
-  unsigned imagedirlen = 0;
-  /** thumbdir length */
-  unsigned thumbdirlen = 0;
 
   srcinfo.err = jpeg_std_error (&jsrcerr.pub);
   jsrcerr.pub.addon_message_table = exif_message_table;
@@ -1908,17 +2041,8 @@ main (int argc, char **argv)
     exif_output_message;
 
   for (argv++; argc > 1; argv++, argc--) {
-    struct jpeg_transform_info info;
-    struct jvirt_barray_control** src_coef_arrays;
-    struct jvirt_barray_control** dst_coef_arrays;
     time_t datetime;
     const char* opts = ignopt || **argv != '-' ? 0 : *argv;
-    /** destination file name */
-    char* dstname = 0;
-    /** input or output file */
-    FILE* f;
-    /** flag: is this a JFIF image (not Exif) */
-    int jfif = 0;
 
     if (opts && !strcmp (opts, "--")) {
       argv++;
@@ -1930,53 +2054,55 @@ main (int argc, char **argv)
     while (opts && *++opts) { /* process all flags */
       switch (*opts) {
       default:
-	putc ('-', stderr);
-	putc (*opts, stderr);
-	fputs (": unknown option (use -h for help)\n", stderr);
+	(void) putc ('-', stderr);
+	(void) putc (*opts, stderr);
+	(void) fputs (": unknown option (use -h for help)\n", stderr);
 	status = 1;
 	goto done;
       case '?':
       case 'h':
-	fputs ("exifiron 1.2\n"
-	       "Orientation correction and shrinking of Exif files\n"
-	       "Usage: exifiron [options] file ...\n"
-	       "Options:\n"
-	       "-h"
-	       "\tusage\n"
-	       "-0"
-	       "\tautomatic orientation correction (default)\n"
-	       "-1,-u"
-	       "\tno transformation\n"
-	       "-2,-f"
-	       "\thorizontal flip\n"
-	       "-3,-U"
-	       "\tupside down\n"
-	       "-4,-F"
-	       "\tvertical flip\n"
-	       "-5,-t"
-	       "\ttranspose\n"
-	       "-6,-R"
-	       "\trotate right\n"
-	       "-7,-T"
-	       "\ttransverse\n"
-	       "-8,-r"
-	       "\trotate left\n"
-	       "-e"
-	       "\tremove thumbnail (default)\n"
-	       "-E"
-	       "\tpreserve thumbnail\n"
-	       "-d,-D,-O"
-	       "\tstamp with DateTime{,Digitized,Original} (default=-d)\n"
-	       "-o"
-	       "\tstamp with current time\n"
-	       "-b"
-	       "\tbaseline JPEG\n"
-	       "-p"
-	       "\tprogressive JPEG (default)\n", stderr);
-	fputs ("-s dir\n"
-	       "\tsave the image files in dir instead of replacing originals\n"
-	       "-S dir\n"
-	       "\tsave the Exif thumbnails in dir\n", stderr);
+	(void) fputs
+	  ("exifiron 1.2.1\n"
+	   "Orientation correction and shrinking of Exif files\n"
+	   "Usage: exifiron [options] file ...\n"
+	   "Options:\n"
+	   "-h"
+	   "\tusage\n"
+	   "-0"
+	   "\tautomatic orientation correction (default)\n"
+	   "-1,-u"
+	   "\tno transformation\n"
+	   "-2,-f"
+	   "\thorizontal flip\n"
+	   "-3,-U"
+	   "\tupside down\n"
+	   "-4,-F"
+	   "\tvertical flip\n"
+	   "-5,-t"
+	   "\ttranspose\n"
+	   "-6,-R"
+	   "\trotate right\n"
+	   "-7,-T"
+	   "\ttransverse\n"
+	   "-8,-r"
+	   "\trotate left\n"
+	   "-e"
+	   "\tremove thumbnail (default)\n"
+	   "-E"
+	   "\tpreserve thumbnail\n"
+	   "-d,-D,-O"
+	   "\tstamp with DateTime{,Digitized,Original} (default=-d)\n"
+	   "-o"
+	   "\tstamp with current time\n"
+	   "-b"
+	   "\tbaseline JPEG\n"
+	   "-p"
+	   "\tprogressive JPEG (default)\n", stderr);
+	(void) fputs
+	  ("-s dir\n"
+	   "\tsave the image files in dir instead of replacing originals\n"
+	   "-S dir\n"
+	   "\tsave the Exif thumbnails in dir\n", stderr);
 	goto done;
 
       case '0': transformation = 0; break;
@@ -2003,7 +2129,7 @@ main (int argc, char **argv)
       case 's': case 'S':
 	if (argc <= 2) {
 	  putc ('-', stderr), putc (*opts, stderr);
-	  fputs (": missing argument (use -h for help)\n", stderr);
+	  (void) fputs (": missing argument (use -h for help)\n", stderr);
 	  status = 1;
 	  goto done;
 	}
@@ -2011,9 +2137,9 @@ main (int argc, char **argv)
 	if (!**++argv) {
 	  /* empty string argument => restore default behaviour */
 	  if (*opts == 's')
-	    imagedir = 0, imagedirlen = 0;
+	    imagedir = 0;
 	  else
-	    thumbdir = 0, thumbdirlen = 0;
+	    thumbdir = 0;
 	}
 	else {
 	  unsigned namelen = strlen (*argv);
@@ -2022,30 +2148,32 @@ main (int argc, char **argv)
 	  while (*s) if (*s++ == '\\') s[-1] = '/';
 #endif
 	  if (*opts == 's')
-	    imagedir = *argv, imagedirlen = namelen;
+	    imagedir = *argv;
 	  else
-	    thumbdir = *argv, thumbdirlen = namelen;
+	    thumbdir = *argv;
 	  if (mkdir (*argv
 #if !defined WIN32 && !defined __WIN32
 		     , 0777
 #endif
 		     ) && errno != EEXIST) {
-	    fputs (*argv, stderr);
+	    (void) fputs (*argv, stderr);
 	    perror (": mkdir");
-	    goto quit;
+	  quit:
+	    status = 2;
+	    goto done;
 	  }
 	  else {
 	    struct stat statbuf;
 	    char* s = malloc (namelen + 3);
 	    if (!s) {
 	      putc ('-', stderr), putc (*opts, stderr);
-	      fputs (": out of memory\n", stderr);
+	      (void) fputs (": out of memory\n", stderr);
 	      goto quit;
 	    }
 	    memcpy (s, *argv, namelen);
 	    memcpy (s + namelen, "/.", 3);
 	    if (stat (s, &statbuf)) {
-	      fputs (s, stderr);
+	      (void) fputs (s, stderr);
 	      perror (": stat");
 	      free (s);
 	      goto quit;
@@ -2057,135 +2185,67 @@ main (int argc, char **argv)
       }
     }
 
-    if (opts == *argv + 1)
-      f = stdin; /* file name '-' */
-    else if (opts)
+    if (opts && opts != *argv + 1)
       continue; /* get next parameters */
-    else if (!(f = fopen (*argv, "rb"))) {
-      fputs (*argv, stderr);
-      perror (": open (reading)");
-      continue;
-    }
 
-    inputfile = *argv;
-
-    /* set up jump context for input */
-    if (setjmp (jsrcerr.setjmp_buffer)) {
-      exif_output_message ((j_common_ptr) &srcinfo);
-      jpeg_abort_decompress (&srcinfo);
-      jpeg_abort_compress (&dstinfo);
-      if (f && !opts) fclose (f), f = 0;
-      if (dstname) free (dstname), dstname = 0;
-      continue; /* ignore unreadable or erroneous input files */
-    }
-
-    /* set up jump context for output */
-    if (setjmp (jdsterr.setjmp_buffer)) {
-      if (dstname && errno)
-	perror (dstname);
-      else
-	exif_output_message ((j_common_ptr) &dstinfo);
-      if (f && !opts) fclose (f), f = 0;
-      goto quit2; /* abort on write error */
-    }
-
-    jpeg_stdio_src (&srcinfo, f);
-    jcopy_markers_setup (&srcinfo);
-    jpeg_read_header (&srcinfo, TRUE);
-
-    datetime = tOutput;
-    dstname = thumbdir
-      ? createfilename (*argv, strlen (*argv), thumbdir, thumbdirlen)
-      : 0;
-    info.transform = exif_parse (&srcinfo, transformation,
-				 tOutput == tdtNone ? 0 : &datetime,
-				 rmifd1, dstname);
-    if (dstname) free (dstname), dstname = 0;
-    jfif = !info.transform;
-    if (jfif)
-      info.transform = transformation ? transformation : JXFORM_NONE;
-
-    jtransform_request_workspace (&srcinfo, &info);
-
-    /* Read source file as DCT coefficients */
-    src_coef_arrays = jpeg_read_coefficients (&srcinfo);
-    /* Close the input file */
-    if (opts)
-      f = stdout;
-    else {
-      if (fclose (f)) {
-	fputs (inputfile, stderr);
-	perror ("close");
-      }
-      f = 0;
-    }
-    /* Initialize destination compression parameters from source values */
-    jpeg_copy_critical_parameters (&srcinfo, &dstinfo);
-
-    dst_coef_arrays = jtransform_adjust_parameters (&srcinfo, &dstinfo,
-						    src_coef_arrays,
-						    &info);
-    if (!opts) {
-      int fd;
+    /* if opts != 0 at this point, it denotes the file name '-' */
+    srcname = *argv;
+    {
+      /** destination file name */
+      char* dstname;
+      /** thumbnail file name */
+      char* thumbname;
       /* allocate the destination file name */
       unsigned len = strlen (*argv);
-      if (!(dstname = imagedir
-	    ? createfilename (*argv, len, imagedir, imagedirlen)
-	    : malloc (len + 1))) {
-	fputs (*argv, stderr);
-	fputs (": out of memory\n", stderr);
-	goto quit;
+
+      if (opts) /* file name '-' */
+	dstname = thumbname = 0;
+      else {
+	thumbname = thumbdir
+	  ? createfilename (*argv, len, thumbdir, strlen (thumbdir))
+	  : 0;
+	if (!(dstname = imagedir
+	      ? createfilename (*argv, len, imagedir, strlen (imagedir))
+	      : malloc (len + 1))) {
+	  (void) fputs (*argv, stderr);
+	  (void) fputs (": out of memory\n", stderr);
+	  goto quit2;
+	}
+	if (!imagedir) {
+	  memcpy (dstname, *argv, len + 1);
+	  dstname[len - 1] = '0'; /* change the last character to '0' */
+	}
       }
-      if (!imagedir) {
-	memcpy (dstname, *argv, len + 1);
-	dstname[len - 1] = '0'; /* change the last character to '0' */
-      }
-      unlink (dstname);
-      fd = open (dstname,
-#if defined WIN32 || defined __WIN32
-		 O_WRONLY | O_CREAT | O_EXCL | O_BINARY, 0666
-#else
-		 O_WRONLY | O_CREAT | O_EXCL, 0444
-#endif
-		 );
-      if (fd < 0 || !(f = fdopen (fd, "wb"))) {
-	fputs (dstname, stderr);
-	perror (": open (writing)");
-	goto quit2;
-      }
-    }
-    jpeg_stdio_dest (&dstinfo, f);
-    dstinfo.optimize_coding = TRUE;
-    dstinfo.write_JFIF_header = jfif;
-    if (progressive)
-      jpeg_simple_progression (&dstinfo);
-    jpeg_write_coefficients (&dstinfo, dst_coef_arrays);
-    jcopy_markers_execute (&srcinfo, &dstinfo, thumb_buf, thumb_length);
-    jtransform_execute_transform (&srcinfo, &dstinfo, src_coef_arrays, &info);
-    jpeg_finish_compress (&dstinfo);
-    if (!opts) {
-      if (fclose (f)) {
-	fputs (dstname, stderr);
-	perror ("close");
-	goto quit2;
-      }
-      if (datetime != tOutput)
-	touchfile (dstname, datetime);
-      if (!imagedir && rename (dstname, *argv)) {
-	fputs (dstname, stderr);
-	fputs (" -> ", stderr);
-	fputs (*argv, stderr);
-	perror (": rename");
+      datetime = tOutput;
+      switch (jpeg_transform (&srcinfo, &dstinfo, thumbname, dstname,
+			      transformation, rmifd1,
+			      tOutput == tdtNone ? 0 : &datetime,
+			      progressive)) {
+      case 0:
+	break;
+      case 1:
+	continue;
+      default:
       quit2:
-	free (dstname), dstname = 0;
-      quit:
 	status = 2;
+	if (dstname) free (dstname);
+	if (thumbname) free (thumbname);
 	goto done;
       }
-      free (dstname), dstname = 0;
+      if (dstname) {
+	if (datetime != (time_t) tOutput)
+	  touchfile (dstname, datetime);
+	if (!imagedir && rename (dstname, *argv)) {
+	  (void) fputs (dstname, stderr);
+	  (void) fputs (" -> ", stderr);
+	  (void) fputs (*argv, stderr);
+	  perror (": rename");
+	  goto quit2;
+	}
+      }
+      free (dstname);
+      if (thumbname) free (thumbname);
     }
-    f = 0;
-    (void) jpeg_finish_decompress (&srcinfo);
   }
 
  done:
