@@ -56,15 +56,15 @@ html_uri (FILE* file,
   for (; *s; s++) {
     switch (*s) {
     default:
-      if (*s > 0x20 && *s < 0x7e) {
+      if (*s > (char) 0x20 && *s < (char) 0x7e) {
 	if (putc (*s, file) == EOF)
 	  return 1;
 	break;
       }
-      /* fall through */
+      /*@fallthrough@*/
     case '"': case '#': case '%': case '?': case ':': case '+':
     case '&': case '<': case '>':
-      if (3 != fprintf (file, "%%%02x", (unsigned char) *s))
+      if (3 != fprintf (file, "%%%02x", (unsigned) (unsigned char) *s))
 	return 1;
     }
   }
@@ -245,7 +245,6 @@ html_uriref (FILE* file,
 /** Generate links to parent directories
  * @param file	the output stream
  * @param dir	the directory name
- * @param base	the base name of dir
  * @param ixnam	index.html file name
  * @param spec	the thumbnail directory name
  * @return	zero on success, nonzero on failure
@@ -253,7 +252,6 @@ html_uriref (FILE* file,
 static int
 html_parent_links (FILE* file,
 		   const char* dir,
-		   const char* base,
 		   const char* ixnam,
 		   const char* spec)
 {
@@ -397,7 +395,7 @@ dumphtml (FILE* file,
 		" content=\"text/html; charset=\"") ||
       html_raw (file, opts->charset) ||
       html_raw (file, "\">\n"
-		"<meta name=\"Generator\" content=\"pHoToMoLo 1.2;\n "
+		"<meta name=\"Generator\" content=\"pHoToMoLo 1.2.1;\n "
 		"http://www.funet.fi/pub/sci/graphics/packages/photomolo/"
 		"photomolo.html\">\n"
 		"<link rel=\"start\" href=\"") ||
@@ -426,7 +424,7 @@ dumphtml (FILE* file,
       html_raw (file,
 		"</head>\n\n"
 		"<body>\n<h1>") ||
-      html_parent_links (file, rootname, title, opts->html, spec->thumbdir) ||
+      html_parent_links (file, rootname, opts->html, spec->thumbdir) ||
       html_esc (file, title, 0) ||
       html_raw (file, "</h1>\n"
 		"<table><tr>\n"
@@ -442,7 +440,7 @@ dumphtml (FILE* file,
   if (!(fn = malloc (fnlen + speclen + 1))) {
   memory:
     free (fn);
-    fputs ("out of memory\n", stderr);
+    (void) fputs ("out of memory\n", stderr);
     return 1;
   }
   /* prepare the thumbnail file name */
@@ -491,20 +489,20 @@ dumphtml (FILE* file,
       fn = s;
       memcpy (fn + fnlen + speclen, tree->files[i].name, namelen);
       if (!(f = fopen (fn, "rb"))) {
-	fputs (fn, stderr);
+	(void) fputs (fn, stderr);
 	perror (": open (reading)");
 	goto error;
       }
       s = (char*) readinfo (&thumb_info, 0, f);
-      fclose (f);
+      (void) fclose (f);
       if (s) {
-	fputs (fn, stderr);
-	fputs (": ", stderr), fputs (s, stderr);
-	putc ('\n', stderr);
+	(void) fputs (fn, stderr);
+	(void) fputs (": ", stderr), (void) fputs (s, stderr);
+	(void) putc ('\n', stderr);
       }
       else
-	fprintf (file, "\nwidth=\"%u\" height=\"%u\"",
-		 thumb_info.width, thumb_info.height);
+	(void) fprintf (file, "\nwidth=\"%u\" height=\"%u\"",
+			thumb_info.width, thumb_info.height);
     }
 
     if (html_raw (file, "></a>\n"))
