@@ -59,14 +59,14 @@ touch_file (const char* name,
   int status;
   struct stat statbuf;
   if ((status = stat (ref, &statbuf))) {
-    fputs (ref, stderr);
+    (void) fputs (ref, stderr);
     perror (": stat");
   }
   else {
     struct utimbuf utim;
     utim.actime = utim.modtime = statbuf.st_mtime;
     if ((status = utime (name, &utim))) {
-      fputs (ref, stderr);
+      (void) fputs (ref, stderr);
       perror (": utime");
     }
   }
@@ -105,10 +105,10 @@ exif_output_message (struct jpeg_common_struct* cinfo)
   /** error message buffer for libjpeg */
   static char jpeg_msg[JMSG_LENGTH_MAX];
   cinfo->err->format_message (cinfo, jpeg_msg);
-  fputs (srcname, stderr);
-  fputs (": ", stderr);
-  fputs (jpeg_msg, stderr);
-  putc ('\n', stderr);
+  (void) fputs (srcname, stderr);
+  (void) fputs (": ", stderr);
+  (void) fputs (jpeg_msg, stderr);
+  (void) putc ('\n', stderr);
 }
 
 /** error messages from the Exif parser */
@@ -136,7 +136,11 @@ enum exif_message {
  * @param cinfo	the compression context
  */
 static void
-thumb_out_dummy (struct jpeg_compress_struct* cinfo)
+thumb_out_dummy (struct jpeg_compress_struct* cinfo
+#ifdef __GNUC__
+		 __attribute__ ((unused))
+#endif /* __GNUC__ */
+		 )
 {
 }
 
@@ -300,34 +304,34 @@ copy_markers (struct jpeg_decompress_struct* srcinfo,
    * if the encoder library already wrote one.
    */
   for (marker = srcinfo->marker_list; marker != NULL; marker = marker->next) {
-    if (marker->marker == JPEG_APP0 &&
+    if (marker->marker == (UINT8) JPEG_APP0 &&
 	marker->data_length >= 5 &&
-	GETJOCTET (marker->data[0]) == 0x4A &&
-	GETJOCTET (marker->data[1]) == 0x46 &&
-	((GETJOCTET (marker->data[2]) == 0x49 &&
-	  GETJOCTET (marker->data[3]) == 0x46) ||
-	 (GETJOCTET (marker->data[2]) == 0x58 &&
-	  GETJOCTET (marker->data[3]) == 0x58)) &&
-	GETJOCTET (marker->data[4]) == 0)
+	GETJOCTET (marker->data[0]) == (JOCTET) 0x4A &&
+	GETJOCTET (marker->data[1]) == (JOCTET) 0x46 &&
+	((GETJOCTET (marker->data[2]) == (JOCTET) 0x49 &&
+	  GETJOCTET (marker->data[3]) == (JOCTET) 0x46) ||
+	 (GETJOCTET (marker->data[2]) == (JOCTET) 0x58 &&
+	  GETJOCTET (marker->data[3]) == (JOCTET) 0x58)) &&
+	GETJOCTET (marker->data[4]) == (JOCTET) 0)
       continue; /* reject JFIF and JFXX markers */
     if (dstinfo->write_Adobe_marker &&
-	marker->marker == JPEG_APP0 + 14 &&
+	marker->marker == (UINT8) (JPEG_APP0 + 14) &&
 	marker->data_length >= 5 &&
-	GETJOCTET (marker->data[0]) == 0x41 &&
-	GETJOCTET (marker->data[1]) == 0x64 &&
-	GETJOCTET (marker->data[2]) == 0x6F &&
-	GETJOCTET (marker->data[3]) == 0x62 &&
-	GETJOCTET (marker->data[4]) == 0x65)
+	GETJOCTET (marker->data[0]) == (JOCTET) 0x41 &&
+	GETJOCTET (marker->data[1]) == (JOCTET) 0x64 &&
+	GETJOCTET (marker->data[2]) == (JOCTET) 0x6F &&
+	GETJOCTET (marker->data[3]) == (JOCTET) 0x62 &&
+	GETJOCTET (marker->data[4]) == (JOCTET) 0x65)
       continue; /* reject duplicate Adobe */
     if (thumb_length &&
-	marker->marker == JPEG_APP0 + 1 &&
+	marker->marker == (UINT8) (JPEG_APP0 + 1) &&
 	marker->data_length >= 8 &&
-	GETJOCTET (marker->data[0]) == 0x45 &&
-	GETJOCTET (marker->data[1]) == 0x78 &&
-	GETJOCTET (marker->data[2]) == 0x69 &&
-	GETJOCTET (marker->data[3]) == 0x66 &&
-	!GETJOCTET (marker->data[4]) &&
-	!GETJOCTET (marker->data[5])) {
+	GETJOCTET (marker->data[0]) == (JOCTET) 0x45 &&
+	GETJOCTET (marker->data[1]) == (JOCTET) 0x78 &&
+	GETJOCTET (marker->data[2]) == (JOCTET) 0x69 &&
+	GETJOCTET (marker->data[3]) == (JOCTET) 0x66 &&
+	GETJOCTET (marker->data[4]) == (JOCTET) 0x00 &&
+	GETJOCTET (marker->data[5]) == (JOCTET) 0x00) {
       /* writing Exif marker with embedded thumbnail */
       jpeg_write_m_header (dstinfo, marker->marker, marker->data_length +
 			   94 + thumb_length);
@@ -381,8 +385,8 @@ static unsigned
 exif_read2 (JOCTET FAR* data)
 {
   return exif_endian
-    ? GETJOCTET (data[0]) << 8 | GETJOCTET (data[1])
-    : GETJOCTET (data[0]) | GETJOCTET (data[1]) << 8;
+    ? (unsigned) GETJOCTET (data[0]) << 8 | GETJOCTET (data[1])
+    : (unsigned) GETJOCTET (data[0]) | GETJOCTET (data[1]) << 8;
 }
 
 /** Read a 4-octet quantity from the Exif record
@@ -417,27 +421,27 @@ exif_parse (struct jpeg_decompress_struct* cinfo)
     /** current image file directory (IFD) offset */
     unsigned long offset;
 
-    if (marker->marker != JPEG_APP0 + 1 ||
+    if (marker->marker != (UINT8) (JPEG_APP0 + 1) ||
 	marker->data_length < 14)
       continue; /* Exif data is in an APP1 marker of at least 14 octets */
 
-    if (GETJOCTET (marker->data[0]) != 0x45 ||
-	GETJOCTET (marker->data[1]) != 0x78 ||
-	GETJOCTET (marker->data[2]) != 0x69 ||
-	GETJOCTET (marker->data[3]) != 0x66 ||
-	GETJOCTET (marker->data[4]) ||
-	GETJOCTET (marker->data[5]))
+    if (GETJOCTET (marker->data[0]) != (JOCTET) 0x45 ||
+	GETJOCTET (marker->data[1]) != (JOCTET) 0x78 ||
+	GETJOCTET (marker->data[2]) != (JOCTET) 0x69 ||
+	GETJOCTET (marker->data[3]) != (JOCTET) 0x66 ||
+	GETJOCTET (marker->data[4]) != (JOCTET) 0x00 ||
+	GETJOCTET (marker->data[5]) != (JOCTET) 0x00)
       continue; /* no Exif header */
 
-    if (GETJOCTET (marker->data[6]) == 0x4d &&
-	GETJOCTET (marker->data[7]) == 0x4d &&
-	GETJOCTET (marker->data[8]) == 0x00 &&
-	GETJOCTET (marker->data[9]) == 0x2a)
+    if (GETJOCTET (marker->data[6]) == (JOCTET) 0x4d &&
+	GETJOCTET (marker->data[7]) == (JOCTET) 0x4d &&
+	GETJOCTET (marker->data[8]) == (JOCTET) 0x00 &&
+	GETJOCTET (marker->data[9]) == (JOCTET) 0x2a)
       exif_endian = MSB;
-    else if (GETJOCTET (marker->data[6]) == 0x49 &&
-	     GETJOCTET (marker->data[7]) == 0x49 &&
-	     GETJOCTET (marker->data[8]) == 0x2a &&
-	     GETJOCTET (marker->data[9]) == 0x00)
+    else if (GETJOCTET (marker->data[6]) == (JOCTET) 0x49 &&
+	     GETJOCTET (marker->data[7]) == (JOCTET) 0x49 &&
+	     GETJOCTET (marker->data[8]) == (JOCTET) 0x2a &&
+	     GETJOCTET (marker->data[9]) == (JOCTET) 0x00)
       exif_endian = LSB;
     else
       ERREXIT (cinfo, EXIF_UNKNOWN_ENDIAN);
@@ -529,6 +533,141 @@ createfilename (const char* base,
   return fnam;
 }
 
+/** Transform the image
+ * @param srcinfo	image decompression context
+ * @param dstinfo	image compression context
+ * @param dstname	destination image file name (NULL = stdin)
+ * @param progressive	flag: use progressive coding
+ * @return		0=ok, 1=read error, 2=write error
+ */
+static int
+jpeg_transform (struct jpeg_decompress_struct* srcinfo,
+		struct jpeg_compress_struct* dstinfo,
+		const char* dstname,
+		int progressive)
+{
+  /** input file */
+  static FILE* src = 0;
+  /** output file */
+  static FILE* dst = 0;
+  /* set up jump context for input */
+  if (setjmp (((struct exif_error_mgr*) srcinfo->err)->setjmp_buffer)) {
+    if (errno)
+      perror (dstname ? srcname : "(standard input)");
+    else
+      exif_output_message ((j_common_ptr) srcinfo);
+  ignore:
+    jpeg_abort_decompress (srcinfo);
+    jpeg_abort_compress (dstinfo);
+    if (src) (void) fclose (src), src = 0;
+    if (dst) (void) fclose (dst), dst = 0;
+    return 1;
+  }
+
+  /* set up jump context for output */
+  if (setjmp (((struct exif_error_mgr*) dstinfo->err)->setjmp_buffer)) {
+    if (errno)
+      perror (dstname ? dstname : "(standard output)");
+    else
+      exif_output_message ((j_common_ptr) dstinfo);
+    if (src) (void) fclose (src), src = 0;
+    if (dst) (void) fclose (dst), dst = 0;
+    return 2;
+  }
+
+  if (dstname && !(src = fopen (srcname, "rb"))) {
+    (void) fputs (srcname, stderr);
+    perror (": open (reading)");
+    return 1;
+  }
+  jpeg_stdio_src (srcinfo, dstname ? src : stdin);
+  /* save COM markers */
+  jpeg_save_markers (srcinfo, JPEG_COM, 0xFFFF);
+  /* Save APP0..APP15 markers */
+  {
+    unsigned m;
+    for (m = JPEG_APP0; m < JPEG_APP0 + 16; m++)
+      jpeg_save_markers (srcinfo, m, 0xFFFF);
+  }
+  (void) jpeg_read_header (srcinfo, TRUE);
+  exif_parse (srcinfo);
+  /* create thumbnail image for JFIF or Exif files */
+  if ((thumb_width || thumb_height) &&
+      (exif_endian != UNKNOWN || srcinfo->saw_JFIF_marker)) {
+    thumb_create (srcinfo);
+    if (!thumb_length)
+      goto ignore; /* thumbnail too large */
+    /* re-read the file after constructing the thumbnail */
+    if (fseek (src, 0, SEEK_SET)) {
+      (void) fputs (srcname, stderr);
+      perror (": fseek");
+      goto ignore;
+    }
+    (void) jpeg_finish_decompress (srcinfo);
+    jpeg_stdio_src (srcinfo, src);
+    (void) jpeg_read_header (srcinfo, TRUE);
+    exif_parse (srcinfo); /* need to patch the Exif marker again */
+  }
+  else
+    thumb_length = 0;
+
+  if (dstname) {
+    /* Open the destination file */
+    int fd;
+    (void) unlink (dstname);
+    fd = open (dstname,
+#if defined WIN32 || defined __WIN32
+	       O_WRONLY | O_CREAT | O_EXCL | O_BINARY, 0666
+#else
+	       O_WRONLY | O_CREAT | O_EXCL, 0444
+#endif
+	       );
+    if (fd < 0 || !(dst = fdopen (fd, "wb"))) {
+      (void) fputs (dstname, stderr);
+      perror (": open (writing)");
+      (void) fclose (src);
+      return 2;
+    }
+  }
+  jpeg_stdio_dest (dstinfo, dstname ? dst : stdout);
+  jpeg_copy_critical_parameters (srcinfo, dstinfo);
+  dstinfo->optimize_coding = TRUE;
+  dstinfo->write_JFIF_header =
+    srcinfo->saw_JFIF_marker && exif_endian == UNKNOWN;
+  dstinfo->JFIF_major_version = (UINT8) 1;
+  dstinfo->JFIF_minor_version = (UINT8) 2;
+  if (progressive)
+    jpeg_simple_progression (dstinfo);
+  jpeg_write_coefficients (dstinfo, jpeg_read_coefficients (srcinfo));
+  /* write thumbnail in JFXX marker for JFIF images */
+  if (dstinfo->write_JFIF_header && thumb_length) {
+    /** JFXX marker header for JPEG thumbnail */
+    static const unsigned char jfxx[] =
+      { 0x4A, 0x46, 0x58, 0x58, 0x00, 0x10 };
+    unsigned i;
+    jpeg_write_m_header (dstinfo, JPEG_APP0,
+			 thumb_length + sizeof jfxx);
+    for (i = 0; i < sizeof jfxx; i++)
+      jpeg_write_m_byte (dstinfo, jfxx[i]);
+    for (i = 0; i < thumb_length; i++)
+      jpeg_write_m_byte (dstinfo, thumb_buf[i]);
+  }
+  /* copy other markers (and embed thumbnail in Exif marker) */
+  copy_markers (srcinfo, dstinfo);
+  jpeg_finish_compress (dstinfo);
+  if (src && fclose (src)) {
+    (void) fputs (srcname, stderr);
+    perror ("close");
+    /* ignore error on closing input file */
+  }
+  if (dst && fclose (dst)) {
+    (void) fputs (dstname, stderr);
+    perror ("close");
+    return 2;
+  }
+  return 0;
+}
+
 int
 main (int argc, char **argv)
 {
@@ -565,12 +704,6 @@ main (int argc, char **argv)
 
   for (argv++; argc > 1; argv++, argc--) {
     const char* opts = ignopt || **argv != '-' ? 0 : *argv;
-    /** destination file name */
-    char* dstname = 0;
-    /** input file */
-    FILE* infile = 0;
-    /** output file */
-    FILE* outfile = 0;
 
     if (opts && !strcmp (opts, "--")) {
       argv++;
@@ -582,32 +715,33 @@ main (int argc, char **argv)
     while (opts && *++opts) { /* process all flags */
       switch (*opts) {
       default:
-	putc ('-', stderr);
-	putc (*opts, stderr);
-	fputs (": unknown option (use -h for help)\n", stderr);
+	(void) putc ('-', stderr);
+	(void) putc (*opts, stderr);
+	(void) fputs (": unknown option (use -h for help)\n", stderr);
 	status = 1;
 	goto done;
       case '?':
       case 'h':
-	fputs ("jpegnail 1.2\n"
-	       "Remove or embed thumbnails in Exif or JFIF JPEG images\n"
-	       "Usage: jpegnail [options] file ...\n"
-	       "Options:\n"
-	       "-h"
-	       "\tusage\n"
-	       "-q num"
-	       "\tthumbnail quality (0..100)\n"
-	       "-x num"
-	       "\tmaximum thumbnail width\n"
-	       "-y num"
-	       "\tmaximum thumbnail height\n"
-	       "-b"
-	       "\tbaseline JPEG\n"
-	       "-p"
-	       "\tprogressive JPEG (default)\n"
-	       "-s dir\n"
-	       "\tsave the image files in dir instead of replacing originals\n"
-	       , stderr);
+	(void) fputs
+	  ("jpegnail 1.2.1\n"
+	   "Remove or embed thumbnails in Exif or JFIF JPEG images\n"
+	   "Usage: jpegnail [options] file ...\n"
+	   "Options:\n"
+	   "-h"
+	   "\tusage\n"
+	   "-q num"
+	   "\tthumbnail quality (0..100)\n"
+	   "-x num"
+	   "\tmaximum thumbnail width\n"
+	   "-y num"
+	   "\tmaximum thumbnail height\n"
+	   "-b"
+	   "\tbaseline JPEG\n"
+	   "-p"
+	   "\tprogressive JPEG (default)\n"
+	   "-s dir\n"
+	   "\tsave the image files in dir instead of replacing originals\n",
+	   stderr);
 	goto done;
 
       case 'b': progressive = 0; break;
@@ -616,8 +750,8 @@ main (int argc, char **argv)
       case 's':
 	if (argc <= 2) {
 	missing:
-	  putc ('-', stderr), putc (*opts, stderr);
-	  fputs (": missing argument (use -h for help)\n", stderr);
+	  (void) putc ('-', stderr), (void) putc (*opts, stderr);
+	  (void) fputs (": missing argument (use -h for help)\n", stderr);
 	  status = 1;
 	  goto done;
 	}
@@ -639,7 +773,7 @@ main (int argc, char **argv)
 		     , 0777
 #endif
 		     ) && errno != EEXIST) {
-	    fputs (*argv, stderr);
+	    (void) fputs (*argv, stderr);
 	    perror (": mkdir");
 	    goto quit;
 	  }
@@ -647,14 +781,14 @@ main (int argc, char **argv)
 	    struct stat statbuf;
 	    char* s = malloc (imagedirlen + 3);
 	    if (!s) {
-	      putc ('-', stderr), putc (*opts, stderr);
-	      fputs (": out of memory\n", stderr);
+	      (void) putc ('-', stderr), (void) putc (*opts, stderr);
+	      (void) fputs (": out of memory\n", stderr);
 	      goto quit;
 	    }
 	    memcpy (s, *argv, imagedirlen);
 	    memcpy (s + imagedirlen, "/.", 3);
 	    if (stat (s, &statbuf)) {
-	      fputs (s, stderr);
+	      (void) fputs (s, stderr);
 	      perror (": stat");
 	      free (s);
 	      goto quit;
@@ -674,9 +808,9 @@ main (int argc, char **argv)
 	  argc--;
 	  if (q < 1L || q > 65535L || *endp || !*argv) {
 	  args:
-	    fputc ('-', stderr);
-	    fputc (*opts, stderr);
-	    fputs (": invalid argument\n", stderr);
+	    (void) putc ('-', stderr);
+	    (void) putc (*opts, stderr);
+	    (void) fputs (": invalid argument\n", stderr);
 	    goto quit;
 	  }
 	  if (*opts == 'q') {
@@ -693,160 +827,44 @@ main (int argc, char **argv)
       }
     }
 
-    srcname = *argv;
-    if (opts == *argv + 1)
-      infile = stdin, outfile = stdout; /* file name '-' */
-    else if (opts)
+    if (opts && opts != *argv + 1)
       continue; /* get next parameters */
-    else {
-      if (!(infile = fopen (srcname, "rb"))) {
-	fputs (srcname, stderr);
-	perror (": open (reading)");
-	continue;
-      }
-    }
 
-    /* set up jump context for input */
-    if (setjmp (jsrcerr.setjmp_buffer)) {
-      exif_output_message ((j_common_ptr) &srcinfo);
-    ignore:
-      jpeg_abort_decompress (&srcinfo);
-      jpeg_abort_compress (&dstinfo);
-      if (!opts) {
-	if (infile) fclose (infile), infile = 0;
-	if (outfile) fclose (outfile), outfile = 0;
-      }
-      if (dstname) free (dstname), dstname = 0;
-      status = 2;
-      continue; /* ignore unreadable or erroneous input files */
-    }
-
-    /* set up jump context for output */
-    if (setjmp (jdsterr.setjmp_buffer)) {
-      if (dstname && errno)
-	perror (dstname);
-      else
-	exif_output_message ((j_common_ptr) &dstinfo);
-      if (!opts) {
-	if (infile) fclose (infile), infile = 0;
-	if (outfile) fclose (outfile), outfile = 0;
-      }
-      goto quit; /* abort on write error */
-    }
-
-    jpeg_stdio_src (&srcinfo, infile);
-    /* save COM markers */
-    jpeg_save_markers (&srcinfo, JPEG_COM, 0xFFFF);
-    /* Save APP0..APP15 markers */
+    /* if opts != 0 at this point, it denotes the file name '-' */
+    srcname = *argv;
     {
-      unsigned m;
-      for (m = JPEG_APP0; m < JPEG_APP0 + 16; m++)
-	jpeg_save_markers (&srcinfo, m, 0xFFFF);
-    }
-    jpeg_read_header (&srcinfo, TRUE);
-    exif_parse (&srcinfo);
-    /* create thumbnail image for JFIF or Exif files */
-    if ((thumb_width || thumb_height) &&
-	(exif_endian != UNKNOWN || srcinfo.saw_JFIF_marker)) {
-      thumb_create (&srcinfo);
-      if (!thumb_length)
-	goto ignore; /* thumbnail too large */
-      /* re-read the file after constructing the thumbnail */
-      if (fseek (infile, 0, SEEK_SET)) {
-	fputs (srcname, stderr);
-	perror (": fseek");
-	goto ignore;
+      /** destination file name */
+      char* dstname = 0;
+      if (!opts) { /* file name not '-' */
+	unsigned len = strlen (*argv);
+	if (!(dstname = imagedir
+	      ? createfilename (*argv, len, imagedir, imagedirlen)
+	      : malloc (len + 1))) {
+	  (void) fputs (*argv, stderr);
+	  (void) fputs (": out of memory\n", stderr);
+	  goto quit;
+	}
+	if (!imagedir) {
+	  memcpy (dstname, *argv, len + 1);
+	  dstname[len - 1] = '0'; /* change the last character to '0' */
+	}
       }
-      (void) jpeg_finish_decompress (&srcinfo);
-      jpeg_stdio_src (&srcinfo, infile);
-      jpeg_read_header (&srcinfo, TRUE);
-      exif_parse (&srcinfo); /* need to patch the Exif marker again */
-    }
-    else
-      thumb_length = 0;
-
-    if (!opts) {
-      /* Open the destination file */
-      int fd;
-      unsigned len = strlen (*argv);
-      if (!(dstname = imagedir
-	    ? createfilename (*argv, len, imagedir, imagedirlen)
-	    : malloc (len + 1))) {
-	fputs (*argv, stderr);
-	fputs (": out of memory\n", stderr);
-	fclose (infile);
-	goto quit;
-      }
-      if (!imagedir) {
-	memcpy (dstname, *argv, len + 1);
-	dstname[len - 1] = '0'; /* change the last character to '0' */
-      }
-      unlink (dstname);
-      fd = open (dstname,
-#if defined WIN32 || defined __WIN32
-		 O_WRONLY | O_CREAT | O_EXCL | O_BINARY, 0666
-#else
-		 O_WRONLY | O_CREAT | O_EXCL, 0444
-#endif
-		 );
-      if (fd < 0 || !(outfile = fdopen (fd, "wb"))) {
-	fputs (dstname, stderr);
-	perror (": open (writing)");
-	fclose (infile);
-	goto quit;
+      jpeg_transform (&srcinfo, &dstinfo, dstname, progressive);
+      if (!opts) {
+	(void) touch_file (dstname, srcname);
+	if (!imagedir && rename (dstname, srcname)) {
+	  (void) fputs (dstname, stderr);
+	  (void) fputs (" -> ", stderr);
+	  (void) fputs (srcname, stderr);
+	  perror (": rename");
+	quit:
+	  free (dstname), dstname = 0;
+	  status = 3;
+	  goto done;
+	}
+	free (dstname);
       }
     }
-    jpeg_stdio_dest (&dstinfo, outfile);
-    jpeg_copy_critical_parameters (&srcinfo, &dstinfo);
-    dstinfo.optimize_coding = TRUE;
-    dstinfo.write_JFIF_header =
-      srcinfo.saw_JFIF_marker && exif_endian == UNKNOWN;
-    dstinfo.JFIF_major_version = 1;
-    dstinfo.JFIF_minor_version = 2;
-    if (progressive)
-      jpeg_simple_progression (&dstinfo);
-    jpeg_write_coefficients (&dstinfo, jpeg_read_coefficients (&srcinfo));
-    /* write thumbnail in JFXX marker for JFIF images */
-    if (dstinfo.write_JFIF_header && thumb_length) {
-      /** JFXX marker header for JPEG thumbnail */
-      static const unsigned char jfxx[] =
-	{ 0x4A, 0x46, 0x58, 0x58, 0x00, 0x10 };
-      unsigned i;
-      jpeg_write_m_header (&dstinfo, JPEG_APP0,
-			   thumb_length + sizeof jfxx);
-      for (i = 0; i < sizeof jfxx; i++)
-	jpeg_write_m_byte (&dstinfo, jfxx[i]);
-      for (i = 0; i < thumb_length; i++)
-	jpeg_write_m_byte (&dstinfo, thumb_buf[i]);
-    }
-    /* copy other markers (and embed thumbnail in Exif marker) */
-    copy_markers (&srcinfo, &dstinfo);
-    jpeg_finish_compress (&dstinfo);
-    if (!opts) {
-      if (fclose (infile)) {
-	fputs (srcname, stderr);
-	perror ("close");
-	/* ignore error on input file */
-      }
-      if (fclose (outfile)) {
-	fputs (dstname, stderr);
-	perror ("close");
-	goto quit;
-      }
-      touch_file (dstname, srcname);
-      if (!imagedir && rename (dstname, srcname)) {
-	fputs (dstname, stderr);
-	fputs (" -> ", stderr);
-	fputs (srcname, stderr);
-	perror (": rename");
-      quit:
-	free (dstname), dstname = 0;
-	status = 3;
-	goto done;
-      }
-      free (dstname), dstname = 0;
-    }
-    infile = outfile = 0;
     (void) jpeg_finish_decompress (&srcinfo);
   }
 
