@@ -100,7 +100,7 @@ make_thumbnails (const struct dirtree* dir,
   if (!(fn = malloc (fnlen + 1))) {
   memory:
     free (fn);
-    fputs ("out of memory\n", stderr);
+    (void) fputs ("out of memory\n", stderr);
     return 1;
   }
 
@@ -120,7 +120,7 @@ make_thumbnails (const struct dirtree* dir,
     fn = s;
     memcpy (fn + fnlen, dir->files[i].name, namelen);
     if (!(input = fopen (fn, "rb"))) {
-      fputs (fn, stderr);
+      (void) fputs (fn, stderr);
       perror (": open");
       free (fn);
       return 2;
@@ -129,14 +129,14 @@ make_thumbnails (const struct dirtree* dir,
     for (spec = thumbs; spec; spec = spec->next) {
       unsigned speclen = strlen (spec->thumbdir) + 1;
       if (!(s = realloc (fn, fnlen + namelen + speclen))) {
-	fclose (input);
+	(void) fclose (input);
 	goto memory;
       }
       fn = s;
       memcpy (fn + fnlen, spec->thumbdir, speclen);
       if (make_directory (fn)) {
       errexit:
-	fclose (input);
+	(void) fclose (input);
 	free (fn);
 	return 3;
       }
@@ -145,7 +145,7 @@ make_thumbnails (const struct dirtree* dir,
       if (!exists_mtime (fn, dir->files[i].mtime)) {
 	FILE* output = fopen (fn, "wb");
 	if (!output) {
-	  fputs (fn, stderr);
+	  (void) fputs (fn, stderr);
 	  perror (": open (writing)");
 	  goto errexit;
 	}
@@ -154,11 +154,11 @@ make_thumbnails (const struct dirtree* dir,
 	cout.image_height = spec->max_height;
 	if (scale_image (&cin, &cout, spec->shrink, quality) ||
 	    touch_mtime (fn, dir->files[i].mtime)) {
-	  fclose (output);
+	  (void) fclose (output);
 	  goto errexit;
 	}
 	if (fclose (output)) {
-	  fputs (fn, stderr);
+	  (void) fputs (fn, stderr);
 	  perror (": close");
 	  goto errexit;
 	}
@@ -168,7 +168,7 @@ make_thumbnails (const struct dirtree* dir,
     }
     if (fclose (input)) {
       memcpy (fn + fnlen, dir->files[i].name, namelen);
-      fputs (fn, stderr);
+      (void) fputs (fn, stderr);
       perror (": close"); /* report but otherwise ignore the error */
     }
   }
@@ -203,7 +203,7 @@ dumphtmls (const struct dirtree* tree,
   char* filename = malloc (dirnamelen + 1);
   if (!filename) {
   memory:
-    fputs ("out of memory\n", stderr);
+    (void) fputs ("out of memory\n", stderr);
     return 1;
   }
   memcpy (filename, tree->name, dirnamelen - 1);
@@ -231,7 +231,7 @@ dumphtmls (const struct dirtree* tree,
     filename[dirnamelen + speclen - 1] = '/';
     memcpy (filename + dirnamelen + speclen, hopts.html, indexnamelen);
     if (!(f = fopen (filename, "wb"))) {
-      fputs (filename, stderr);
+      (void) fputs (filename, stderr);
       perror (": open");
       free (filename);
       return 1;
@@ -239,7 +239,7 @@ dumphtmls (const struct dirtree* tree,
     status = dumphtml (f, tree, level, info, comm, &hopts, thumbs, spec,
 		       top, prev, next);
     if (fclose (f) || status) {
-      fputs (filename, stderr);
+      (void) fputs (filename, stderr);
       perror (status ? ": write" : ": close");
       if (!status)
 	status = -1;
@@ -282,7 +282,7 @@ dumpfiles (const struct dirtree* tree,
   filename = malloc (dirnamelen + 2);
   if (!filename) {
   memory:
-    fputs ("out of memory\n", stderr);
+    (void) fputs ("out of memory\n", stderr);
     return 1;
   }
   memcpy (filename, tree->name, dirnamelen);
@@ -305,24 +305,24 @@ dumpfiles (const struct dirtree* tree,
       FILE* f;
       len = strlen (tree->files[i].name) + 1;
       if (!(s = realloc (filename, dirnamelen + len))) {
-	fputs ("out of memory\n", stderr);
+	(void) fputs ("out of memory\n", stderr);
 	status = 1;
 	break; /* out of memory */
       }
       filename = s;
       memcpy (filename + dirnamelen, tree->files[i].name, len);
       if (!(f = fopen (filename, "rb"))) {
-	fputs (filename, stderr);
+	(void) fputs (filename, stderr);
 	perror (": fopen");
 	continue; /* ignore unreadable files */
       }
       s = (char*) readinfo (info + i, comment + i, f);
-      fclose (f);
+      (void) fclose (f);
       if (s) {
 	/* ignore errors while reading image info */
-	fputs (filename, stderr);
-	fputs (": ", stderr), fputs (s, stderr);
-	putc ('\n', stderr);
+	(void) fputs (filename, stderr);
+	(void) fputs (": ", stderr), (void) fputs (s, stderr);
+	(void) putc ('\n', stderr);
 	freeinfo (comment[i]); comment[i] = 0;
 	/* clear the info structure for this file */
 	memset (info + i, 0, sizeof *info);
@@ -431,8 +431,8 @@ create_css (const char* path)
     return 0; /* absolute location */
   pathlen = strlen (path);
   if (!(s = malloc (pathlen + 1 + cssnamelen))) {
-    fputs (path, stderr);
-    fputs (": out of memory\n", stderr);
+    (void) fputs (path, stderr);
+    (void) fputs (": out of memory\n", stderr);
     return 1;
   }
   memcpy (s, path, pathlen);
@@ -441,22 +441,22 @@ create_css (const char* path)
   if ((f = fopen (s, "rt")))
     goto done;
   if (!(f = fopen (s, "wt"))) {
-    fputs (s, stderr);
+    (void) fputs (s, stderr);
     perror (": open (writing)");
     free (s);
     return 2;
   }
   if ((sizeof css1) - 1 != fwrite (css1, 1, (sizeof css1) - 1, f) ||
       (sizeof css2) - 1 != fwrite (css2, 1, (sizeof css2) - 1, f)) {
-    fputs (s, stderr);
+    (void) fputs (s, stderr);
     perror (": write");
     free (s);
-    fclose (f);
+    (void) fclose (f);
     return 2;
   }
  done:
   free (s);
-  fclose (f);
+  (void) fclose (f);
   return 0;
 }
 
@@ -471,39 +471,41 @@ main (int argc, char** argv)
   jpeg_create_decompress (&cin);
   jpeg_create_compress (&cout);
 
-  atexit (cleanup);
-  setlocale (LC_ALL, "");
+  (void) atexit (cleanup);
+  (void) setlocale (LC_ALL, "");
 
   if (argc == 1) {
   Usage:
-    fputs ("pHoToMoLo 1.2\n"
-	   "Create thumbnails and HTML navigation\n"
-	   "Usage: photomolo [options] directory1 directory2 ...\n"
-	   "Options:\n"
-	   "-h"
-	   "\tusage\n"
-	   "-T <dir>/<spec>\n"
-	   "\tgenerate thumbnail directory <dir> according to <spec>\n"
-	   "-H htmlfile\n"
-	   "\tname of HTML files (default=\"index.html\")\n"
-	   "-C cssfile\n"
-	   "\tname of CSS files (default=\"index.css\")\n",
-	   stderr);
-    fputs ("-c charset\n"
-	   "\tHTML and JPEG comment character set (default=\"iso-8859-1\")\n"
-	   "-i iconfile\n"
-	   "-I file.ico\n"
-	   "\tname of icon files (default=none)\n"
-	   "-t"
-	   "\tgenerate thumbnails only (no HTML or CSS)\n"
-	   "-r"
-	   "\tremove the thumbnail directories listed with -T\n"
-	   "-q quality\n"
-	   "\tJPEG compression quality (default=75)\n"
-	   "-d"
-	   "\tomit timestamps from the HTML\n"
-	   "--\tdisable processing further options\n",
-	   stderr);
+    (void) fputs
+      ("pHoToMoLo 1.2.1\n"
+       "Create thumbnails and HTML navigation\n"
+       "Usage: photomolo [options] directory1 directory2 ...\n"
+       "Options:\n"
+       "-h"
+       "\tusage\n"
+       "-T <dir>/<spec>\n"
+       "\tgenerate thumbnail directory <dir> according to <spec>\n"
+       "-H htmlfile\n"
+       "\tname of HTML files (default=\"index.html\")\n"
+       "-C cssfile\n"
+       "\tname of CSS files (default=\"index.css\")\n",
+       stderr);
+    (void) fputs
+      ("-c charset\n"
+       "\tHTML and JPEG comment character set (default=\"iso-8859-1\")\n"
+       "-i iconfile\n"
+       "-I file.ico\n"
+       "\tname of icon files (default=none)\n"
+       "-t"
+       "\tgenerate thumbnails only (no HTML or CSS)\n"
+       "-r"
+       "\tremove the thumbnail directories listed with -T\n"
+       "-q quality\n"
+       "\tJPEG compression quality (default=75)\n"
+       "-d"
+       "\tomit timestamps from the HTML\n"
+       "--\tdisable processing further options\n",
+       stderr);
     return 0;
   }
 
@@ -519,16 +521,16 @@ main (int argc, char** argv)
     while (*++opts) { /* process all flags */
       switch (*opts) {
       default:
-	putc ('-', stderr);
-	putc (*opts, stderr);
-	fputs (": unknown option (use -h for help)\n", stderr);
+	(void) putc ('-', stderr);
+	(void) putc (*opts, stderr);
+	(void) fputs (": unknown option (use -h for help)\n", stderr);
 	return 1;
       case '?':
       case 'h':
 	goto Usage;
       case 'T':
 	if (argc <= 2) {
-	  fputs ("-T: missing argument (use -h for help)\n", stderr);
+	  (void) fputs ("-T: missing argument (use -h for help)\n", stderr);
 	  return 1;
 	}
 	else {
@@ -548,35 +550,37 @@ main (int argc, char** argv)
 	break;
       case 'H':
 	if (argc <= 2) {
-	  fputs ("-H: missing argument (use -h for help)\n", stderr);
+	  (void) fputs ("-H: missing argument (use -h for help)\n", stderr);
 	  return 1;
 	}
 	argc--;
 	if (!*(hopts.html = *++argv)) {
-	  fputs ("-H: argument must be nonempty (use -h for help)\n", stderr);
+	  (void) fputs ("-H: argument must be nonempty (use -h for help)\n",
+			stderr);
 	  return 1;
 	}
 	else if (strchr (hopts.html, '/')) {
-	  fputs ("-H ", stderr);
-	  fputs (hopts.html, stderr);
-	  fputs (": name must not contain '/'", stderr);
+	  (void) fputs ("-H ", stderr);
+	  (void) fputs (hopts.html, stderr);
+	  (void) fputs (": name must not contain '/'", stderr);
 	  return 1;
 	}
 	break;
       case 'C':
 	if (argc <= 2) {
-	  fputs ("-C: missing argument (use -h for help)\n", stderr);
+	  (void) fputs ("-C: missing argument (use -h for help)\n", stderr);
 	  return 1;
 	}
 	argc--;
 	if (!*(hopts.css = *++argv)) {
-	  fputs ("-C: argument must be nonempty (use -h for help)\n", stderr);
+	  (void) fputs ("-C: argument must be nonempty (use -h for help)\n",
+			stderr);
 	  return 1;
 	}
 	break;
       case 'c':
 	if (argc <= 2) {
-	  fputs ("-c: missing argument (use -h for help)\n", stderr);
+	  (void) fputs ("-c: missing argument (use -h for help)\n", stderr);
 	  return 1;
 	}
 	argc--;
@@ -584,7 +588,7 @@ main (int argc, char** argv)
 	break;
       case 'i':
 	if (argc <= 2) {
-	  fputs ("-i: missing argument (use -h for help)\n", stderr);
+	  (void) fputs ("-i: missing argument (use -h for help)\n", stderr);
 	  return 1;
 	}
 	argc--;
@@ -593,7 +597,7 @@ main (int argc, char** argv)
 	break;
       case 'I':
 	if (argc <= 2) {
-	  fputs ("-I: missing argument (use -h for help)\n", stderr);
+	  (void) fputs ("-I: missing argument (use -h for help)\n", stderr);
 	  return 1;
 	}
 	argc--;
@@ -611,7 +615,7 @@ main (int argc, char** argv)
 	break;
       case 'q':
 	if (argc <= 2) {
-	  fputs ("-q: missing argument (use -h for help)\n", stderr);
+	  (void) fputs ("-q: missing argument (use -h for help)\n", stderr);
 	  return 1;
 	}
 	else {
@@ -619,7 +623,7 @@ main (int argc, char** argv)
 	  unsigned long q = strtoul (*++argv, &endp, 0);
 	  argc--;
 	  if (q < 1L || q > 100L || *endp || !*argv) {
-	    fputs ("-q: quality must be between 1 and 100\n", stderr);
+	    (void) fputs ("-q: quality must be between 1 and 100\n", stderr);
 	    return 1;
 	  }
 	  quality = q;
@@ -629,7 +633,8 @@ main (int argc, char** argv)
   }
 
   if (!thumbs) {
-    fputs ("expected at least one -T parameter (use -h for help)\n", stderr);
+    (void) fputs ("expected at least one -T parameter (use -h for help)\n",
+		  stderr);
     return 1;
   }
 
