@@ -25,15 +25,37 @@
 
 #include "jpeglib.h"
 
+/** Prepare to scale a JPEG image
+ * @param cin		the image to be scaled
+ * @param cout		the output image
+ *			(image_width and image_height set to the maximum,
+ *			0=unspecified)
+ * @param shrink	shrink factor (0=unspecified, 1, 2, 4 or 8)
+ */
+void
+scale_prepare (struct jpeg_decompress_struct* cin,
+	       struct jpeg_compress_struct* cout,
+	       unsigned shrink);
+
+/** Copy or resize a JPEG image
+ * @param cin		the input image
+ * @param cout		the output (with exact image_width and image_height)
+ */
+void
+scale_jpeg (struct jpeg_decompress_struct* cin,
+	    struct jpeg_compress_struct* cout);
+
 /** Scale a JPEG image
  * @param cin		the image to be scaled
  * @param cout		the output image
- *			(image_width and image_width set to the maximum,
+ *			(image_width and image_height set to the maximum,
  *			0=unspecified)
  * @param shrink	shrink factor (0=unspecified, 1, 2, 4 or 8)
+ * @param quality	JPEG compression quality (1..100, libjpeg default=75)
  * @return		0 on success, nonzero on failure
  */
 int
 scale_image (struct jpeg_decompress_struct* cin,
 	     struct jpeg_compress_struct* cout,
-	     unsigned shrink);
+	     unsigned shrink,
+	     int quality);
