@@ -85,7 +85,7 @@ readtree (struct dirtree* tree,
   unsigned i;
 
   if (!(dir = opendir (tree->name))) {
-    fputs (tree->name, stderr);
+    (void) fputs (tree->name, stderr);
     perror (": opendir");
     return 1;
   }
@@ -94,8 +94,8 @@ readtree (struct dirtree* tree,
   i = strlen (tree->name);
   if (!(pathname = malloc (i + 1))) {
   memory0:
-    fputs (tree->name, stderr);
-    fputs (": out of memory\n", stderr);
+    (void) fputs (tree->name, stderr);
+    (void) fputs (": out of memory\n", stderr);
     return 2;
   }
   memcpy (pathname, tree->name, i);
@@ -118,7 +118,7 @@ readtree (struct dirtree* tree,
       memcpy (pathname + i, dirent->d_name, namelen);
     }
     if (stat (pathname, &statbuf)) {
-      fputs (pathname, stderr);
+      (void) fputs (pathname, stderr);
       perror (": stat");
     }
     else if (S_ISDIR (statbuf.st_mode)) {
@@ -129,8 +129,8 @@ readtree (struct dirtree* tree,
 	  realloc (tree->dirs, (tree->n_dirs + 1) * (2 * sizeof *dirs));
 	if (!dirs) {
 	memory:
-	  fputs (pathname, stderr);
-	  fputs (": out of memory\n", stderr);
+	  (void) fputs (pathname, stderr);
+	  (void) fputs (": out of memory\n", stderr);
 	  free (pathname);
 	  freetree (tree);
 	  return 2;
@@ -143,7 +143,7 @@ readtree (struct dirtree* tree,
       /* regular file; is it a readable image file? */
       FILE* f = fopen (pathname, "rb");
       if (!f) {
-	fputs (pathname, stderr);
+	(void) fputs (pathname, stderr);
 	perror (": open (reading)");
       other:
 	/* non-image file */
@@ -167,7 +167,7 @@ readtree (struct dirtree* tree,
 	int is_jpeg =
 	  getc (f) == 0xff &&
 	  getc (f) == 0xd8;
-	fclose (f);
+	(void) fclose (f);
 	if (!is_jpeg)
 	  goto other;
       }
@@ -188,7 +188,7 @@ readtree (struct dirtree* tree,
     }
   }
 
-  closedir (dir);
+  (void) closedir (dir);
   free (pathname);
 
   /* process the subdirectories */
@@ -270,7 +270,7 @@ touch_mtime (const char* path,
   int status;
   utim.actime = utim.modtime = mtime;
   if ((status = utime (path, &utim))) {
-    fputs (path, stderr);
+    (void) fputs (path, stderr);
     perror (": utime");
   }
   return status;
@@ -292,7 +292,7 @@ make_directory (const char* path)
 	      ) ||
       errno == EEXIST)
     return 0;
-  fputs (path, stderr);
+  (void) fputs (path, stderr);
   perror (": mkdir");
   return 1;
 }
@@ -316,7 +316,7 @@ remove_directory (const char* path)
   if (!rmdir (path) || errno == ENOENT)
     return 0; /* directory does not exist, good enough */
   if (!(dir = opendir (path))) {
-    fputs (path, stderr);
+    (void) fputs (path, stderr);
     perror (": opendir");
     return 1;
   }
@@ -325,8 +325,8 @@ remove_directory (const char* path)
   i = strlen (path);
   if (!(pathname = malloc (i + 1))) {
   memory0:
-    fputs (path, stderr);
-    fputs (": remove_directory: out of memory\n", stderr);
+    (void) fputs (path, stderr);
+    (void) fputs (": remove_directory: out of memory\n", stderr);
     return 2;
   }
   memcpy (pathname, path, i);
@@ -346,7 +346,7 @@ remove_directory (const char* path)
       pathname = s;
       memcpy (pathname + i, dirent->d_name, namelen);
       if (unlink (pathname)) {
-	fputs (pathname, stderr);
+	(void) fputs (pathname, stderr);
 	perror (": unlink");
 	free (pathname);
 	return 2;
@@ -354,11 +354,11 @@ remove_directory (const char* path)
     }
   }
 
-  closedir (dir);
+  (void) closedir (dir);
   free (pathname);
 
   if (rmdir (path)) {
-    fputs (path, stderr);
+    (void) fputs (path, stderr);
     perror (": rmdir");
     return 2;
   }
